% Calculate the material stiffness matrix.
%   Call as:
%     D = tangent_moduli(m, context)
%   where
%     m=material
%    context=structure with optional field:
%                kind = string with possible values lambda, 
%                       lambda_shear, bulk, bulk_shear
%
% the output arguments are
%     D=matrix 6x6
%
function D = tangent_moduli(self, context)
    if isfield(context,'kind')
        switch context.kind
            case 'lambda'
                D = tangent_moduli_lambda(self, context);
            case 'lambda_shear'
                D = tangent_moduli_shear(self, context);
            case 'bulk'
                D = tangent_moduli_bulk(self, context);
            case 'bulk_shear'
                D = tangent_moduli_bulk_shear(self, context);
        end
    else
        E = self.E;
        nu = self.nu;
        lambda = E * nu / (1 + nu) / (1 - 2*(nu));
        mu     = E / (2 * (1 + nu));
        mI = diag([1 1 1 0.5 0.5 0.5]);
        m1 = [1 1 1 0 0 0]';
        D = lambda * m1 * m1' + 2 * mu * mI;
    end
    return;
end

% Calculate the part of the material stiffness matrix that corresponds to
% the lambda Lame coefficient.
function D = tangent_moduli_lambda(self, context)
    E = self.E;
    nu = self.nu;
    lambda = E * nu / (1 + nu) / (1 - 2*(nu));
    m1 = [1 1 1 0 0 0]';
    D = lambda * m1 * m1';
    return;
end

% Calculate the part of the material stiffness matrix that correspond to shear.
% Note: makes sense only for isotropic materials.
function D = tangent_moduli_shear(self, context)
    E = self.E;
    nu = self.nu;
    mu     = E / (2 * (1 + nu));
    mI = diag([1 1 1 0.5 0.5 0.5]);
    D = 2 * mu * mI;
    return;
end

% Calculate the part of the material stiffness matrix that corresponds to
% the bulk modulus.
function D = tangent_moduli_bulk(self, context)
    E = self.E;
    nu = self.nu;
    B = E / 3 / (1 - 2*(nu));
    m1 = [1 1 1 0 0 0]';
    D = B * m1 * m1';
    return;
end

% Calculate the part of the material stiffness matrix that correspond to shear.
% Note: makes sense only for isotropic materials.
function D = tangent_moduli_shear_bulk(self, context)
    E = self.E;
    nu = self.nu;
    mu     = E / (2 * (1 + nu));
    D = 2 * mu * [2/3*[2 -1 -1; -1 2 -1; -1 -1 2] zeros(3,3); zeros(3,3) eye(3,3) ];
    return;
end